
#import "Tracker.h"
#import "Actor.h"

@implementation Tracker

double normAngle(double theta)
{
	if (theta > M_PI)
		return normAngle(theta - 2.0*M_PI);
	else if (theta < -M_PI)
		return normAngle(theta + 2.0*M_PI);
	else
		return theta;
}


- initFor:aShip
{
	[super init];
	resolution = 0.1;
	ship = aShip;
	return self;
}

- track:target
{
	to = target;
	return [self update];
}

- ship
{
	return ship;
}

- target
{
	return ship;
}

- setResolution:(float) res
{
	resolution = res;
	return self;
}

- update
{
// Egads this is slopppy; plus it hardwires wrap around!
	if (to) {
		int n = N_AXIS;
		int off[] = {xOffset, yOffset};
		while (n--) {
			x[n] =  [ship distance:to axis:n];
			v[n] =  [ship velocity:to axis:n];
			if (x[n] > off[n])
				x[n] -= 2*off[n];
			else if (x[n] < -off[n])
				x[n] += 2*off[n];
		}
	} else {
		x[0] = x[1] = 0.0;
		v[0] = v[1] = 0.0;
	}
	return self;
}

-(float) angle
{
	register float at = atan2(x[1],x[0]);
	return normAngle(at - [ship angle]);
}

- (float) distance
{
    return sqrt(x[0]*x[0] + x[1]*x[1]);
}

- (float) velocity	
{
    return sqrt(v[0]*v[0] + v[1]*v[1]);
}

-(float) Vangle
{
	register float at = atan2(v[1],v[0]);
	return normAngle(at - [ship angle]);
}


- (BOOL) turnTowards
{
	return [self turnTowards:[self angle]];
}

- (BOOL) turnTowards:(float)angle
{
	if (fabs(angle) < resolution) {
		[ship setTurning:LEFT down:NO time:0.0];
		[ship setTurning:RIGHT down:NO time:0.0];
		return YES;
	} else {
		ROTATION LR = (angle > 0.0) ? LEFT : RIGHT;
		[ship setTurning:LR down:YES time:0.0];
		return NO;
	}
}

- (BOOL) thrustTowards
{
	return [self thrustTowards:[self angle]];
}

- (BOOL) thrustTowards:(float)angle
{
	BOOL isFacing = [self turnTowards:angle];
	[ship setThrusting:isFacing time:0.0];
	return isFacing;
}

- (BOOL) matchVelocity
{
	[self thrustTowards:normAngle(M_PI - [self Vangle])];
	return fabs([self angle]) < resolution;
}

- (BOOL) shieldsWithin:(float)distance
{
	BOOL flag = ([self distance] < distance);
	[ship setShields:flag];
	return flag;
}

- (BOOL) fireBetween:(float)far :(float)near
{
	float dist = [self distance];
	if (dist < far && dist > near) {
		[ship setShields:NO];
		[ship fire];
		return YES;
	}
	return NO;
}

@end

@implementation Ship (Tracker)

- (float) distance:(Actor*)to axis:(AXIS)xy;
{
	return (xy == X_AXIS) ? to->x - x : to->y - y;
}

- (float) velocity:(Actor*)to axis:(AXIS)xy;
{
	return (xy == X_AXIS) ? to->xv - xv : to->yv - yv;
}

- (float) angle
{
	return theta + M_PI_2;
}

@end
